/************************************************************************/
/*									*/
/*	ng_split - splits Norton Guides database into components	*/
/*									*/
/************************************************************************/
/*									*/
/*	ng_split, Copyright (C) 1988, by John C. Gordon			*/
/*	Norton Guides Splitter, 18 July 1988, ALL RIGHTS RESERVED	*/
/*									*/
/*	This program is meant for those of us who bought the Norton	*/
/*	Guides and were disappointed by the fact that we couldn't	*/
/*	modify the databases provided for our own purposes (by making	*/
/*	the fatal assumption that "customizing the Guides" included	*/
/*	the ability to customize the published databases).		*/
/*									*/
/*	I work in Turbo C occassionally, but work with minicomputer	*/
/*	and mainframe versions of UNIX and C most of the time, using	*/
/*	the PC as a dial-up terminal.  I had planned on installing the	*/
/*	Norton Guides on my PC and modifying them for the specific	*/
/*	flavors of UNIX and C that I work with.  Since there was no	*/
/*	"un-compiler" in the package, I had to either write the C	*/
/*	databases from scratch, or figure out how to turn the database	*/
/*	back into source text.  This program is the result.		*/
/*									*/
/*	I have been able to take the databases provided in the Guides,	*/
/*	"un-compile" (split) them into their component parts, and	*/
/*	then re-compile them back into a database using the NGC and	*/
/*	NGML utilities provided with the Guides.  The resulting		*/
/*	database is identical to the original using DOS 'comp'		*/
/*	(except for a six-character difference in the Microsoft C	*/
/*	database, which does not appear to affect execution).		*/
/*									*/
/*	I have tested this program on the Norton Guides for Turbo C	*/
/*	Microsoft C.  Others have tested it on the other published	*/
/*	databases and some public domain ones.  If you find that it	*/
/*	does not work for some database, I will be glad to look at	*/
/*	the code to see if I can get it to work.  I will need a copy	*/
/*	of the database file, however (only for debugging - the		*/
/*	disk(s) will be returned with a copy of the new program).	*/
/*									*/
/*	If you have any suggestions for this program or others,		*/
/*	please feel free to write to me.				*/
/*	If you like this program and would like to see more like	*/
/*	it, please contribute whatever you think this program is	*/
/*	worth to you (recommended $5 or $10).  Thank you.		*/
/*									*/
/*			John C. Gordon					*/
/*			Post Office Box 25107				*/
/*			Alexandria, VA  22313-5107			*/
/*									*/
/*			Home phone : (703) 528-2205			*/
/*									*/
/************************************************************************/

/************************************************************************/
/*									*/
/*	ng_split		Version 1.1		18 July 1988	*/
/*									*/
/*	Purpose:	Split a Norton Guides .ng database into		*/
/*			its original component source files		*/
/*									*/
/*	Syntax:		ng_split db_name				*/
/*	Where:		'db_name' = the name of the Norton Guides	*/
/*			database (with or without the .ng suffix)	*/
/*									*/
/*	Restrictions:	1)  The NG database must be located in your	*/
/*			    current working directory.			*/
/*			2)  The database must not be "active" when	*/
/*			    you run ng_split.  You must either		*/
/*			    uninstall the Guides or copy the database	*/
/*			    to a different (work) directory first.	*/
/*			3)  Due to the intensive character I/O, you	*/
/*			    should run from your RAMdisk, if possible.	*/
/*									*/
/*	Remarks:	ng_split will create several files in your	*/
/*			current working directory :			*/
/*									*/
/*		'db_name':	the menu link control file		*/
/*		'db_name'.bat:	a batch file containing the commands	*/
/*			to re-create the database			*/
/*		'dbxxx'_00:	the individual source data files are	*/
/*			named 'dbxxx'_00 through 'dbxxx'_99, depending	*/
/*			on how many files are needed.  Note : 'dbxxx'	*/
/*			is the first five characters of 'db_name'.	*/
/*		temp:	a temporary file which is deleted on exit	*/
/*									*/
/*	For example : if you are splitting the Turbo C database tc.ng,	*/
/*		you would use the command:	ng_split tc		*/
/*		which would create files:	tc			*/
/*						tc.bat			*/
/*						tc_00 through tc_11	*/
/*						temp (deleted)		*/
/*									*/
/*	Change history:							*/
/*		V 1.0 - 17 June 88 - original program			*/
/*		V 1.1 - 18 July 88 - allow databases with names longer	*/
/*				than six characters or with more than	*/
/*				26 source files.			*/
/*									*/
/************************************************************************/

#include <stdio.h>

long i;

get_n (n, stream)
int n;
FILE *stream;
{
	while (n-- > 0) { getc (stream); i++; }
	return;
}
   
get_cvt (stream)
FILE *stream;
{
	int n;

	n = getc (stream); i++;
	if ((n % 32) >= 16) n = n - 16; else n = n + 16;
	if ((n % 16) >= 8)  n = n - 8;  else n = n + 8;
	if ((n % 4)  >= 2)  n = n - 2;  else n = n + 2;
	return (n);
}
   
get_2num (stream)
FILE *stream;
{
	int n;

	n = get_cvt (stream) + (get_cvt (stream) * 256);
	return (n);
}
   
long get_4addr (stream)
FILE *stream;
{
	long n;

	n  =  (long) get_cvt (stream);
	n += ((long) get_cvt (stream) * 256L);
	n += ((long) get_cvt (stream) * 256L * 256L);
	n += ((long) get_cvt (stream) * 256L * 256L * 256L);
	return (n);
}

main (argc, argv)
int argc;
char *argv[];
{
	int zc, n, pn, j, ct, cz, len, items, z, addr_ct, cp;
	int cr, zlen, tlen, see_ct, m, m1, m2, temp_file, temp_ch;
	long addr[100], seealso[100];

	char name_in[13], name_out[13], name_bat[13];
	FILE *in, *out, *bat, *temp;

	printf ("\nNG_SPLIT, Copyright (C) 1988, by John C. Gordon\n");
	printf ("Norton Guides Splitter, 18 July 1988, ALL RIGHTS RESERVED\n\n");
	
	if (argc == 0)
		{ printf ("You need to supply a database name !\n"); exit (1); }

	j = 0;
	while (*argv[1] != '\0' && *argv[1] != '.')
	{
		name_in[j] = *argv[1];
		name_bat[j] = *argv[1];
		name_out[j++] = *argv[1]++;
	}
	name_in[j]  = '.'; name_in[j+1]  = 'n'; name_in[j+2]  = 'g';
	name_in[j+3]  = '\0';
	name_bat[j]  = '.'; name_bat[j+1]  = 'b'; name_bat[j+2]  = 'a';
	name_bat[j+3]  = 't'; name_bat[j+4]  = '\0';
	name_out[j] = '\0';

	if ((in = fopen (name_in, "rb")) == NULL)
		{ printf ("cannot open %s for input\n", name_in); goto close; }
	if ((bat = fopen (name_bat, "wb")) == NULL)
		{ printf ("cannot open %s for output\n", name_bat); goto close; }
	if ((out = fopen (name_out, "wb")) == NULL)
		{ printf ("cannot open %s for output\n", name_out); goto close; }

	name_in[j] = '\0';
	if (j > 5) j = 5;
	name_bat[j] = '\0';
	name_out[j] = '_'; name_out[j+1] = '0'; name_out[j+2] = '0';
	name_out[j+3] = '\0';

/* read the header */

	cr = 0;
	fprintf (out, "!name: ");
	for (i = 0; i <= 377; i++)
	{
		n = getc (in);
		if (i < 8) ;
		else if (i < 48)
			{ if (n != 0) putc (n, out); }
		else if (i == 48)
			{ fprintf (out, "\r\n!credits:\r\n%c", n); }
		else if (n == 0) cr++;
		else if (cr > 0)
			{ fprintf (out, "\r\n"); putc (n, out); cr = 0; }
		else putc (n, out);
	}
	fprintf (out, "\r\n");

/* read the menu sections */

	addr_ct = 0; zc = 0;
	n = get_cvt (in); i--;
	while (n == 2)
	{
		get_n (1, in);
		len = get_2num (in);
		items = get_2num (in);

		get_n (20, in);
		for (ct = 1; ct < items; ct++) addr[addr_ct++] = get_4addr (in);
		get_n (items * 8, in);

		z = 0;
		fprintf (out, "\r\n!menu: ");
		cp = 7;
		for (ct = ((items - 1) * 4) + (items * 8); ct < len - 2; ct++)
		{
			n = get_cvt (in);
			if (n == 0)
			{
				if (z != 0)
				{
					while (cp++ < 40) putc (' ', out);
					fprintf (out, " %s_%c%c.ngo",
						name_bat, '0' + (zc / 10),
							  '0' + (zc % 10));
					fprintf (bat, "ngc %s_%c%c\r\n",
						name_bat, '0' + (zc / 10),
							  '0' + (zc % 10));
					zc++;
				}
				fprintf (out, "\r\n       ");
				z = 1; cp = 7;
			}
			else { putc (n, out); cp++; }
		}
		while (cp++ < 40) putc (' ', out);
		fprintf (out, " %s_%c%c.ngo", name_bat, '0' + (zc / 10),
							'0' + (zc % 10));
		fprintf (bat, "ngc %s_%c%c\r\n", name_bat, '0' + (zc / 10),
							   '0' + (zc % 10));
		zc++;
		fprintf (out, "\r\n");
		get_n (2, in);
		n = get_cvt (in);
	}
	fprintf (bat, "ngml %s\r\n", name_in);
	fclose (bat);

/* now < i > = the address of the byte just read into < n >  = first NGO */

	if (i != addr[0]) goto error;

	addr[addr_ct] = addr[addr_ct - 1] * 100L;
	for (z = 1; z <= addr_ct; z++)
	{
		fclose (out);
		if ((out = fopen (name_out, "wb")) == NULL)
		{
			printf ("cannot open %s for output\n", name_out);
			goto close;
		}
		(name_out[j+2])++;
		if (name_out[j+2] > '9')
		{
			name_out[j+2] = '0';
			(name_out[j+1])++;
		}
		temp_file = 0;

		while (i < addr[z] && !feof (in))
		{
			if (n == 0)
			{
				if ((temp = fopen ("temp", "wb")) == NULL)
				{
					printf ("cannot open temp for output\n");
					goto close;
				}
				get_n (1, in);
				tlen = get_2num (in);
				zlen = get_2num (in);
				get_n (20 + (zlen * 6), in);
				n = get_cvt (in);
				fprintf (temp, "!short:");
				for (ct = zlen * 6; ct < tlen - 1; ct++)
				{
					if (n == 0) fprintf (temp, "\r\n!short:");
					else if (n != 255) putc (n, temp);
					else
					{
						n = get_cvt (in); ct++;
						while (n-- > 0) putc (' ', temp);
					}
					n = get_cvt (in);
				}
				fprintf (temp, "\r\n");
				fclose (temp);
				if ((temp = fopen ("temp", "rb")) == NULL)
				{
					printf ("cannot open temp for input\n");
					exit (1);
				}
				temp_ch = getc (temp);
				temp_file = 1;
				n = get_cvt (in);
				if (n != 1) goto error;
			}

			else if (n == 1)
			{
				if (temp_file == 1)
				{
					while (temp_ch != '\r')
					{
						putc (temp_ch, out);
						temp_ch = getc (temp);
					}
					temp_ch = getc (temp);
					temp_ch = getc (temp);
					fprintf (out, "\r\n");
				}
				get_n (1, in);
				zlen = get_2num (in);
				get_n (2, in);
				tlen = get_2num (in);
				if (tlen == 0) tlen = zlen;
				get_n (18, in);
				n = get_cvt (in);
				pn = 0;
				for (ct = 0; ct < tlen; ct++)
				{
					if (n == 0)
					{
						if (pn == 0) putc (' ', out);
						fprintf (out, "\r\n");
					}
					else if (n != 255) putc (n, out);
					else
					{
						n = get_cvt (in); ct++;
						while (n-- > 0) putc (' ', out);
					}
					pn = n; n = get_cvt (in);
				}

/* this page is un-tabbed three tab stops for readability */

	if (tlen != zlen)
	{
		see_ct = n;
		zlen = zlen - tlen - (n * 4) - 2 - 2;
		get_n (1, in);
		fprintf (out, "!seealso: ");
		if (see_ct != 0)
		{
			for (ct = 0; ct < see_ct; ct++)
				seealso[ct] = get_4addr (in);
			m1 = 0;
			for (m2 = addr_ct - 1; seealso[m1] < addr[m2]; m2--) ;
			if (	((name_out[j+1] - '0') * 10)
				+ (name_out[j+2] - '0') != m2 + 1)
				fprintf (out, "%s_%c%c.ngo:",
					name_bat, '0' + (m2 / 10),
						  '0' + (m2 % 10));
			fprintf (out, "\"");
			n = get_cvt (in);
			for (ct = 0; ct < zlen; ct++)
			{
				if (n == 0)
				{
					m1++;
					for (m2 = addr_ct - 1;
						seealso[m1] < addr[m2]; m2--) ;
					fprintf (out, "\" ");
					if (	((name_out[j+1] - '0') * 10)
						+ (name_out[j+2] - '0')
							!= m2 + 1)
						fprintf (out, "%s_%c%c.ngo:",
						    name_bat, '0' + (m2 / 10),
							      '0' + (m2 % 10));
					fprintf (out, "\"");
				}
				else putc (n, out);
				n = get_cvt (in);
			}
			fprintf (out, "\"");
		}
		fprintf (out, "\r\n");
		get_n (1, in);
		n = get_cvt (in);
	}

/* normal tabbing resumes on this page */

			}
			else goto error;
		}
		if (z < addr_ct) if (i > addr[z]) goto error;
		if (z == addr_ct) if (!feof (in)) goto error;
		if (temp_file == 1)
			{ if (!feof (temp)) goto error; fclose (temp); }
	}
	if (ferror (out)) { printf ("Possibly out of disk ! "); goto error; }
	printf ("SUCCESSFUL - Created %d source files !\n", addr_ct);
	goto close;

error:	printf ("ERROR !\n");

close:	fcloseall ();
	unlink ("temp");
	exit (0);
}
